import logging

from pygendata.ddl import DDL
from pygendata.managers import csvmanager, tsvmanager, jsonmanager

class DataGenerator:
    """
    DataGenerator takes a manager of type (csv, tsv, json ..ect)
    DataGenerator takes an optional argument for number of rows to generate when dealing with csv
    """
    def __init__(self, manager, **kwargs):
        if kwargs.get('rows'):
            self.rows = kwargs['rows']
        else:
            self.rows = []
        if manager == 'csv':
            self.manager = csvmanager.CSVManager()
        elif manager == 'tsv':
            self.manager = tsvmanager.TSVManager()
        elif manager == 'json':
            self.manager = jsonmanager.JSONManager()

    def ddl(self, infile, outfile):
        """
        reads a ddl file from disk and generates a csv based on the column names
        """
        try:
            statement = self.manager.read(infile)
            ddl = DDL(statement)
            ddl.get_columns()
            ddl.create_headers()
            self.manager.headers = ddl.headers
            for _ in range(self.rows):
                ddl.create_row()
            self.manager.rows = ddl.column_data
            self.manager.write(outfile)
        except IOError as e:
            logging.warn(str(e))


from pygendata.datatypes import datatypes
from pygendata.exceptions import TypeNotSupportedError

class DDL:
    def __init__(self, statement):
        self.statement = statement
        self.columns = []
        self.column_data = [] # {'name': <column_name>, 'type': <column_type> }
        self.headers = []

    @property
    def column_data(self):
        return self._column_data
    
    @column_data.setter
    def column_data(self, column_data):
        self._column_data = column_data
    
    def get_columns(self):
        _, *cols = self.statement.split('\n')
        cols = [x.rstrip(',') for x in cols]
        cols = [x for x in cols if x != ');'] # remove );
        cols = list(filter(None, cols)) # remove empty strings
        self.columns = cols
    
    def create_headers(self):
        for column in self.columns:
            name, *_ = column.split(' ')
            self.headers.append(name)

    def create_row(self):
        c = {}
        for column in self.columns:
            name, *type_info = column.split(' ')
            type_info = ''.join(type_info)
            if type_info in datatypes:
                c[name] = datatypes[type_info]
            else:
                raise TypeNotSupportedError(f"{c['type']} is not currently supported")
        self.column_data.append(c)
    
    def process(self):
        self.get_columns()
        self.get_column_types()